import type { Moment } from "moment";

export const ALL_TIME = "ALL_TIME";

export interface DateFilterStore {
	filterYear: number | typeof ALL_TIME;
	get filterYears(): number[];
	filterMonth: number | typeof ALL_TIME;
	get filterMonths(): { label: string; value: number }[];
}

export function createDateFilter<T>(
	data: () => T[],
	selector: (item: T) => Moment,
	initialMonth?: boolean
): DateFilterStore & {
	filteredData: T[];
} {
	const today = new Date();
	let filterYear: number | typeof ALL_TIME = $state(today.getFullYear());
	let filterMonth: number | typeof ALL_TIME = $state(
		initialMonth ? today.getMonth() + 1 : ALL_TIME
	);
	const filteredData = $derived.by(() => {
		return data().filter((item) => {
			const date = selector(item);
			if (filterYear !== ALL_TIME) {
				if (date.year() !== filterYear) {
					return false;
				}
			}

			if (filterMonth !== ALL_TIME) {
				if (date.month() !== filterMonth - 1) {
					return false;
				}
			}
			return true;
		});
	});

	const filterYears = $derived.by(() => {
		const years = new Set<number>();
		data().forEach((item) => {
			years.add(selector(item).year());
		});
		return Array.from(years).sort((a, b) => b - a);
	});

	const filterMonths = $derived.by(() => {
		if (filterYear === ALL_TIME) {
			return [];
		}

		const months: { label: string; value: number }[] = [];
		data().forEach((item) => {
			const date = selector(item);
			if (date.year() === filterYear) {
				months.push({
					label: date.format("MMMM"),
					value: date.month() + 1,
				});
			}
		});

		return months
			.filter(
				(month, idx, self) =>
					idx === self.findIndex((m) => m.value === month.value)
			)
			.sort((a, b) => a.value - b.value);
	});

	return {
		get filterYear() {
			return filterYear;
		},
		set filterYear(value) {
			filterYear = value;
		},
		get filterYears() {
			return filterYears;
		},
		get filterMonth() {
			return filterMonth;
		},
		set filterMonth(value) {
			filterMonth = value;
		},
		get filterMonths() {
			return filterMonths;
		},
		get filteredData() {
			return filteredData;
		},
	};
}
