import chroma from "chroma-js";

const style = getComputedStyle(document.body);

export const WCAG_TEXT_CONTRAST_RATIO = 4.5;
export const WCAG_NON_TEXT_CONTRAST_RATIO = 3.5;
export const COLOR_NAMES = [
	"red",
	"orange",
	"yellow",
	"green",
	"cyan",
	"blue",
	"purple",
	"pink",
] as const;
export type ColorName = (typeof COLOR_NAMES)[number];

export function isColorName(color: string): color is ColorName {
	return COLOR_NAMES.includes(color as ColorName);
}

export class Color {
	constructor(
		private readonly _chroma: chroma.Color,
		private readonly _alpha: number = 1
	) {}

	get hex(): string {
		return this.chroma.hex();
	}

	get rgb(): string {
		return this.chroma.css();
	}

	get rgba(): string {
		return this.chroma.alpha(this._alpha).css();
	}

	get chroma(): chroma.Color {
		return this._chroma;
	}

	get contrastColor(): Color {
		return chroma.contrast(this.chroma, "white") < WCAG_TEXT_CONTRAST_RATIO
			? new Color(chroma("black"))
			: new Color(chroma("white"));
	}

	alpha(alpha: number): Color {
		alpha = Math.max(0, Math.min(1, alpha));
		return new Color(this.chroma, alpha);
	}

	darken(amount?: number): Color {
		return new Color(this.chroma.darken(amount));
	}

	lighten(amount?: number): Color {
		return new Color(this.chroma.brighten(amount));
	}

	static fromName(color: ColorName): Color {
		return new Color(chroma(style.getPropertyValue(`--color-${color}`)));
	}

	static fromCSSColor(color: string): Color {
		return new Color(chroma(color));
	}

	static getAll(): Color[] {
		return COLOR_NAMES.map((color) => Color.fromName(color));
	}

	static scale(n: number): Color[] {
		const colors = COLOR_NAMES.map((color) =>
			style.getPropertyValue(`--color-${color}`)
		);

		return chroma
			.scale(colors)
			.mode("lch")
			.colors(n, null)
			.map((color) => new Color(color));
	}
}
